@@ -442,6 +442,8 @@ cal_backend_wcap_events_update_objects (ECalBackendSync *backend, EDataCal *cal,
 
 	if (!cal_backend_wcap_is_online (wcap))
 		return GNOME_Evolution_Calendar_RepositoryOffline;
+	if (!cal_backend_wcap_get_connection (wcap))
+		return cal_backend_wcap_get_open_error (wcap);
 
 	toplevel_comp = icalparser_parse_string (calobj);
 	if (!toplevel_comp)
@@ -595,6 +597,8 @@ cal_backend_wcap_events_remove_object (ECalBackendSync *backend, EDataCal *cal,
 
 	if (!cal_backend_wcap_is_online (wcap))
 		return GNOME_Evolution_Calendar_RepositoryOffline;
+	if (!cal_backend_wcap_get_connection (wcap))
+		return cal_backend_wcap_get_open_error (wcap);
 
 	status = E_CAL_BACKEND_SYNC_CLASS (parent_class)->get_object_sync (backend, cal, uid, rid, &calobj);
 	if (status != GNOME_Evolution_Calendar_Success)
@@ -640,6 +644,9 @@ get_fb_info (SunOneConnection *so_cnc, const char *user, const char *calid, time
 	guint error_code;
 	const char *user_calid;
 
+	if (!so_cnc)
+		return; 
+
 	utc_zone = icaltimezone_get_utc_timezone ();
 
 	if (strchr (user, '@')) {
@@ -699,6 +706,8 @@ cal_backend_wcap_events_get_freebusy (ECalBackendSync *backend, EDataCal *cal, G
 
 	if (!cal_backend_wcap_is_online (wcap))
 		return GNOME_Evolution_Calendar_RepositoryOffline;
+	if (!cal_backend_wcap_get_connection (wcap))
+		return cal_backend_wcap_get_open_error (wcap);
 
 	if (!users) {
 		get_fb_info (cal_backend_wcap_get_connection (wcap), cal_backend_wcap_get_calid (wcap), cal_backend_wcap_get_calid (wcap), start, end, freebusy);
@@ -727,6 +736,8 @@ cal_backend_wcap_events_receive_objects (ECalBackendSync *backend, EDataCal *cal
 
 	if (!cal_backend_wcap_is_online (wcap))
 		return GNOME_Evolution_Calendar_RepositoryOffline;
+	if (!cal_backend_wcap_get_connection (wcap))
+		return cal_backend_wcap_get_open_error (wcap);
 
 	toplevel_comp = icalparser_parse_string (calobj);
 	if (!toplevel_comp)
