@@ -135,12 +135,15 @@ cal_backend_wcap_tasks_update_objects (ECalBackendSync *backend, EDataCal *cal,
 	ECalComponent *comp;
 	icalcomponent *toplevel_comp, *subcomp;
 	icalcomponent_kind kind;
+	gboolean hasImport = FALSE;
 	gboolean relativeAlarm = FALSE;
 
 	g_return_val_if_fail (IS_CAL_BACKEND_WCAP (wcap), GNOME_Evolution_Calendar_OtherError);
 
 	if (!cal_backend_wcap_is_online (wcap))
 		return GNOME_Evolution_Calendar_RepositoryOffline;
+	if (!cal_backend_wcap_get_connection (wcap))
+		return cal_backend_wcap_get_open_error (wcap);
 
 	toplevel_comp = icalparser_parse_string (calobj);
 	if (!toplevel_comp)
@@ -179,6 +182,12 @@ cal_backend_wcap_tasks_update_objects (ECalBackendSync *backend, EDataCal *cal,
 			/* Fix up the component for the server */
 			process_tasks_component_to_server (wcap, comp, &relativeAlarm, &allday);
 
+			if (cal_backend_wcap_need_import (wcap, comp)) {
+				result = cal_backend_wcap_handle_import (wcap, comp, mod, toplevel_comp, subcomp, child_kind);
+				subcomp = icalcomponent_get_next_component (toplevel_comp, ICAL_ANY_COMPONENT);
+				hasImport = TRUE;
+				continue;
+			}
 			/* Get the uid for verification */
 			e_cal_component_get_uid (comp, &uid);
 			
@@ -218,7 +227,7 @@ cal_backend_wcap_tasks_update_objects (ECalBackendSync *backend, EDataCal *cal,
 	g_object_unref (G_OBJECT (comp));
 	icalcomponent_free (toplevel_comp);
 
-	if (relativeAlarm)
+	if (hasImport || relativeAlarm)
 		cal_backend_wcap_poll_cb (wcap);
 
 	return result;
@@ -272,6 +281,8 @@ cal_backend_wcap_tasks_remove_object (ECalBackendSync *backend, EDataCal *cal, c
 
 	if (!cal_backend_wcap_is_online (wcap))
 		return GNOME_Evolution_Calendar_RepositoryOffline;
+	if (!cal_backend_wcap_get_connection (wcap))
+		return cal_backend_wcap_get_open_error (wcap);
 
 	status = E_CAL_BACKEND_SYNC_CLASS (parent_class)->get_object_sync (backend, cal, uid, rid, &calobj);
 	if (status != GNOME_Evolution_Calendar_Success)
@@ -325,6 +336,8 @@ cal_backend_wcap_tasks_receive_objects (ECalBackendSync *backend, EDataCal *cal,
 
 	if (!cal_backend_wcap_is_online (wcap))
 		return GNOME_Evolution_Calendar_RepositoryOffline;
+	if (!cal_backend_wcap_get_connection (wcap))
+		return cal_backend_wcap_get_open_error (wcap);
 
 	toplevel_comp = icalparser_parse_string (calobj);
 	if (!toplevel_comp)
