@@ -27,12 +27,12 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libical/ical.h>
 #include <libedataserver/e-xml-hash-utils.h>
-#include <libedataserver/e-util.h>
-#include "lib/sunone-account.h"
+#include <libedataserver/e-data-server-util.h>
 #include "lib/sunone-ace.h"
 #include "lib/sunone-util.h"
 #include "storage/sunone-component.h"
 #include "storage/sunone-config-listener.h"
+#include "storage/sunone-account.h"
 #include "cal-backend-wcap.h"
 
 struct _CalBackendWCAPPrivate {
@@ -45,6 +45,7 @@ struct _CalBackendWCAPPrivate {
 	SunOneCompType type;
 	CalMode mode;
 	char *cache_name;
+	ECalBackendSyncStatus open_error;
 	
 	gint timeout_id;
 	time_t timestamp;
@@ -80,6 +81,330 @@ static ECalBackendClass *parent_class = NULL;
 
 G_DEFINE_TYPE (CalBackendWCAP, cal_backend_wcap,  E_TYPE_CAL_BACKEND_SYNC)
 
+static gboolean
+compare_dates (ECalComponentDateTime *adt, ECalComponentDateTime *bdt)
+{
+	if (!adt->value && !bdt->value)
+		return TRUE;
+	else if (!adt->value || !bdt->value)
+		return FALSE;
+
+	if ((adt->tzid && !bdt->tzid)
+	    || (!adt->tzid && bdt->tzid))
+		return FALSE;
+	
+	if (adt->tzid && bdt->tzid && strcmp (adt->tzid, bdt->tzid))
+		return FALSE;
+	
+	return !icaltime_compare_date_only (*adt->value, *bdt->value);
+}
+
+static gboolean
+compare_times (ECalComponentDateTime *adt, ECalComponentDateTime *bdt)
+{
+	if (!adt->value && !bdt->value)
+		return TRUE;
+	else if (!adt->value || !bdt->value)
+		return FALSE;
+
+	if ((adt->tzid && !bdt->tzid)
+	    || (!adt->tzid && bdt->tzid))
+		return FALSE;
+	
+	if (adt->tzid && bdt->tzid && strcmp (adt->tzid, bdt->tzid))
+		return FALSE;
+	
+	return (adt->value->hour == bdt->value->hour) 
+		&& (adt->value->minute == bdt->value->minute) 
+		&& (adt->value->second == bdt->value->second);
+}
+
+static gboolean
+start_equal (ECalComponent *acomp, ECalComponent *bcomp, gboolean times)
+{
+	ECalComponentDateTime adt, bdt;
+	gboolean retval;
+	
+	e_cal_component_get_dtstart (acomp, &adt);
+	e_cal_component_get_dtstart (bcomp, &bdt);
+	
+	retval = times ? compare_times (&adt, &bdt) : compare_dates (&adt, &bdt);
+	
+	e_cal_component_free_datetime (&adt);
+	e_cal_component_free_datetime (&bdt);
+
+	return retval;
+}
+
+static gboolean
+end_equal (ECalComponent *acomp, ECalComponent *bcomp, gboolean times)
+{
+	ECalComponentDateTime adt, bdt;
+	gboolean retval;
+	
+	e_cal_component_get_dtend (acomp, &adt);
+	e_cal_component_get_dtend (bcomp, &bdt);
+	
+	retval = times ? compare_times (&adt, &bdt) : compare_dates (&adt, &bdt);
+	
+	e_cal_component_free_datetime (&adt);
+	e_cal_component_free_datetime (&bdt);
+
+	return retval;
+}
+
+static gboolean
+rrules_equal (ECalComponent *acomp, ECalComponent *bcomp)
+{
+	GSList *alist, *blist, *l, *m;
+	gboolean retval = TRUE;
+	
+	/* If neither have rrules, we call them equal */
+	if (!e_cal_component_has_rrules (acomp) && !e_cal_component_has_rrules (bcomp))
+		return TRUE;
+
+	/* If only one has rrules, they are not equal */
+	if ((e_cal_component_has_rrules (acomp) && !e_cal_component_has_rrules (bcomp))
+	    || (!e_cal_component_has_rrules (acomp) && e_cal_component_has_rrules (bcomp)))
+		return FALSE;
+
+	/* Try to match each rrule */
+	e_cal_component_get_rrule_list (acomp, &alist);
+	e_cal_component_get_rrule_list (bcomp, &blist);
+
+	for (l = alist; l != NULL; l = l->next) {
+		struct icalrecurrencetype *a = l->data;
+		gboolean match = FALSE;
+		
+		for (m = blist; m != NULL; m = m->next) {
+			struct icalrecurrencetype *b = m->data;
+			
+			if (!memcmp (a, b, sizeof (struct icalrecurrencetype))) {
+				blist = g_slist_remove_link (blist, m);
+				e_cal_component_free_recur_list (m);
+				match = TRUE;
+				break;
+			}
+		}
+		
+		if (!match) {
+			retval = FALSE;
+			break;
+		}
+	}
+
+	e_cal_component_free_recur_list (alist);
+	e_cal_component_free_recur_list (blist);
+	
+	return retval;
+}
+
+gboolean
+cal_backend_wcap_need_import (CalBackendWCAP *wcap, ECalComponent *comp)
+{
+	ECalComponentOrganizer organizer;
+	gboolean needImport = FALSE;
+
+	e_cal_component_get_organizer (comp, &organizer);
+	if (organizer.value) {
+		if (!strncasecmp ("mailto:", organizer.value, 7)) {
+			/* If we received this via email, we need to import it to the server */
+			needImport = TRUE;
+		} else if (strcasecmp (organizer.value, cal_backend_wcap_get_calid (wcap))) {
+			/* If we are not the organizer and we are not in the attendee list,
+			 * it must be a import */
+			GSList *attendees, *l;
+
+			if (!e_cal_component_has_attendees (comp)) {
+				needImport = TRUE;
+			} else {
+				char *email = g_strconcat ("MAILTO:", cal_backend_wcap_get_account_email (wcap), NULL);
+
+				e_cal_component_get_attendee_list (comp, &attendees);
+				needImport = TRUE;
+				for (l = attendees; l!= NULL; l=l->next) {
+					ECalComponentAttendee *a = l->data;
+
+					if (!strcasecmp (cal_backend_wcap_get_calid (wcap), a->value) ||
+							!strcasecmp (email, a->value)) {
+						needImport = FALSE;
+						break;
+					}
+				}
+				g_free (email);
+			}
+
+		}
+
+	}
+	return needImport;
+}
+
+ECalBackendSyncStatus
+cal_backend_wcap_handle_import (CalBackendWCAP *wcap, ECalComponent *comp, CalObjModType mod, 
+	       icalcomponent *toplevel_comp, icalcomponent *subcomp, icalcomponent_kind child_kind) 
+{
+	ECalComponent *existing;
+	icalcomponent *toplevel_import, *import_comp;
+	icalcompiter iter;
+	icalcomponent_kind import_kind;
+	const char *uid;
+	guint error_code;
+	ECalBackendSyncStatus result = GNOME_Evolution_Calendar_Success;
+
+	e_cal_component_get_uid (comp, &uid);
+							
+	existing = g_hash_table_lookup (cal_backend_wcap_get_objects (wcap), uid);
+
+	/* If its the original item being saved again, it won't have
+	 * the mangled uid so we'll get the first instance */
+	if (!existing && e_cal_component_has_recurrences (comp) && !e_cal_component_is_instance (comp)) {
+		GList *ilist;
+
+		ilist = g_hash_table_lookup (cal_backend_wcap_get_instances (wcap), uid);
+		if (ilist)
+			existing = ilist->data;
+	}
+	
+	/* If it already exists in our hashes, unmangle the uid so we
+	   import it with the correct uid */
+	if (existing && e_cal_component_is_instance (comp))
+		sunone_util_unmangle_uid (comp);
+	e_cal_component_get_uid (comp, &uid);
+
+	toplevel_import = e_cal_util_new_top_level ();
+				
+	iter = icalcomponent_begin_component (toplevel_comp, ICAL_ANY_COMPONENT);
+	while ((import_comp = icalcompiter_deref (&iter))) {
+		import_kind = icalcomponent_isa (import_comp);
+		if (import_kind != child_kind && import_kind != ICAL_VTIMEZONE_COMPONENT) {
+			icalcompiter_next (&iter);
+			continue;
+		}
+
+		if (import_kind == ICAL_VTIMEZONE_COMPONENT) {
+			icalcomponent *clone;
+
+			clone = icalcomponent_new_clone (import_comp);
+			icalcomponent_add_component (toplevel_import, clone);
+		} else if (import_comp == subcomp) {
+			icalcomponent *clone;
+			icalproperty *prop, *prop_clone;
+
+			if (!existing || !e_cal_component_has_recurrences (comp) || mod == CALOBJ_MOD_THIS) {
+				clone = icalcomponent_new_clone (import_comp);
+				icalcomponent_add_component (toplevel_import, clone);
+			} else {
+				GList *ilist, *l;
+				gpointer orig_key;
+				gboolean time_change;
+
+				if (!g_hash_table_lookup_extended (cal_backend_wcap_get_instances (wcap),
+													uid, &orig_key, (gpointer *)&ilist))
+					g_assert_not_reached ();
+
+				if (!start_equal (comp, existing, FALSE) ||
+						!end_equal (comp, existing, FALSE) ||
+						!rrules_equal (comp, existing)) {
+					/* Delete the old ones and let the server re-expand */
+					if (import_kind == ICAL_VEVENT_COMPONENT)
+						sunone_connection_deleteevents_by_id (cal_backend_wcap_get_connection (wcap),
+																cal_backend_wcap_get_calid (wcap),
+																uid, cal_backend_wcap_get_rid_string (existing),
+																cal_backend_wcap_to_sunone_mod (mod, existing));
+					else if (import_kind == ICAL_VTODO_COMPONENT)
+						sunone_connection_deletetodos_by_id (cal_backend_wcap_get_connection (wcap),
+																cal_backend_wcap_get_calid (wcap),
+																uid, cal_backend_wcap_get_rid_string (existing),
+																cal_backend_wcap_to_sunone_mod (mod, existing));
+
+					clone = icalcomponent_new_clone (import_comp);
+					prop = icalcomponent_get_first_property (clone, ICAL_RECURRENCEID_PROPERTY);
+					icalcomponent_remove_property (clone, prop);
+					icalcomponent_add_component (toplevel_import, clone);
+
+					icalcompiter_next (&iter);
+					continue;
+				}
+
+				time_change = !start_equal (comp, existing, TRUE) || !end_equal (comp, existing, TRUE);
+
+				switch (mod) {
+				case CALOBJ_MOD_THISANDPRIOR:
+					break;
+				case CALOBJ_MOD_THISANDFUTURE:
+					ilist = g_list_find_custom (ilist, comp, cal_backend_wcap_instance_list_compare);
+					break;
+				case CALOBJ_MOD_ALL:
+				default:
+					break;
+				}
+
+				for (l = ilist; l; l = l->next) {
+					ECalComponent *recur_comp = l->data;
+					icalcomponent *recur_icalcomp;
+					ECalComponentDateTime dt;
+					struct icaltimetype tt;
+									
+					recur_icalcomp = e_cal_component_get_icalcomponent (recur_comp);
+					clone = icalcomponent_new_clone (import_comp);
+
+					/* Set correct recurrence id */
+					prop = icalcomponent_get_first_property (recur_icalcomp, ICAL_RECURRENCEID_PROPERTY);
+					prop_clone = icalproperty_new_clone (prop);
+									
+					prop = icalcomponent_get_first_property (clone, ICAL_RECURRENCEID_PROPERTY);
+					icalcomponent_remove_property (clone, prop);
+					icalproperty_free (prop);
+					icalcomponent_add_property (clone, prop_clone);
+						
+					/* Preserve the start date (and possibly the time) */
+					e_cal_component_get_dtstart (comp, &dt);
+					prop = icalcomponent_get_first_property (recur_icalcomp, ICAL_DTSTART_PROPERTY);
+					tt = icalproperty_get_dtstart (prop);
+						
+					if (time_change) {
+						/* FIXME handle the case where an instance has a different tz */
+						tt.hour = dt.value->hour;
+						tt.minute = dt.value->minute;
+						tt.second = dt.value->second;
+					}
+					prop = icalcomponent_get_first_property (clone, ICAL_DTSTART_PROPERTY);
+					icalproperty_set_dtstart (prop, tt);
+					e_cal_component_free_datetime (&dt);
+
+					/* Preserve the end date (and possibly the time) */
+					e_cal_component_get_dtend (comp, &dt);
+					prop = icalcomponent_get_first_property (recur_icalcomp, ICAL_DTEND_PROPERTY);
+					tt = icalproperty_get_dtend (prop);
+						
+					if (time_change) {
+						/* FIXME handle the case where an instance has a different tz */
+						tt.hour = dt.value->hour;
+						tt.minute = dt.value->minute;
+						tt.second = dt.value->second;
+					}
+					prop = icalcomponent_get_first_property (clone, ICAL_DTEND_PROPERTY);
+					icalproperty_set_dtend (prop, tt);
+					e_cal_component_free_datetime (&dt);
+
+					icalcomponent_add_component (toplevel_import, clone);
+				}
+			}
+		}
+					
+		icalcompiter_next (&iter);
+	}
+				
+	error_code = sunone_connection_import (cal_backend_wcap_get_connection (wcap), cal_backend_wcap_get_calid (wcap), toplevel_import);
+	if (!SUNONE_ERROR_IS_SUCCESSFUL (error_code))
+		result = cal_backend_wcap_result_from_error (error_code);
+				
+	icalcomponent_free (toplevel_import);
+
+	return result;
+}
+
 SunOneConnection *
 cal_backend_wcap_get_connection (CalBackendWCAP *wcap)
 {
@@ -290,6 +615,7 @@ process_component_from_server (CalBackendWCAP *wcap, ECalComponent *comp)
 	icalcomponent *icalcomp;
 	ECalComponentDateTime dt_start, dt_end;
 	ECalComponentOrganizer organizer;
+	GSList *attendee_list, *l;
 	icaltimezone *izone = NULL;
 	
 	icalcomp = e_cal_component_get_icalcomponent (comp);
@@ -425,6 +751,57 @@ process_component_from_server (CalBackendWCAP *wcap, ECalComponent *comp)
 			g_free (email);
 		}
 	}
+
+	/*if a meeting and account is an attendee, set attendee to account's email address*/
+	e_cal_component_get_attendee_list (comp, &attendee_list);
+	for (l = attendee_list; l != NULL; l = l->next) {
+		ECalComponentAttendee *a = l->data;
+
+		if (!strcasecmp (a->value, priv->calid))
+			break;
+	}
+
+	/*yes, we have to set attendee to account's email address*/
+	if (l != NULL) {
+		GSList *new_al = NULL;
+
+		for (l = attendee_list; l != NULL; l = l->next) {
+			ECalComponentAttendee *a = l->data;
+			ECalComponentAttendee *ca = g_new0 (ECalComponentAttendee, 1);
+
+			if (!strcasecmp (a->value, priv->calid) && (priv->account_email_address != NULL)) {
+				char *email = g_strconcat ("MAILTO:", priv->account_email_address, NULL);
+
+				ca->value = email;
+			} else {
+				if (a->value != NULL)
+					ca->value = g_strdup (a->value);
+			}
+
+			if (a->cn != NULL)
+				ca->cn = g_strdup (a->cn);
+			if (a->member != NULL)
+				ca->member = g_strdup (a->member);
+			if (a->delto != NULL)
+				ca->delto = g_strdup (a->delto);
+			if (a->delfrom != NULL)
+				ca->delfrom = g_strdup (a->delfrom);
+			if (a->sentby != NULL)
+				ca->sentby = g_strdup (a->sentby);
+			if (a->language != NULL)
+				ca->language = g_strdup (a->language);
+			
+			ca->cutype = a->cutype;
+			ca->role = a->role;
+			ca->status = a-> status;
+			ca->rsvp = a->rsvp;
+
+			new_al = g_slist_append (new_al, ca);
+		}
+		e_cal_component_set_attendee_list (comp, new_al);
+		e_cal_component_free_attendee_list (new_al);
+	}
+	e_cal_component_free_attendee_list (attendee_list);
 	
  	/*if there is no attendee, clear organizer*/
 	if(!e_cal_component_has_attendees (comp)) {
@@ -485,9 +862,13 @@ cal_backend_wcap_add_component (CalBackendWCAP *wcap, icalcomponent *icalcomp, g
 	
 	/* Track the mangled uid components */
 	if (g_hash_table_lookup_extended (priv->objects, uid, &orig_key, &orig_value)) {
+		ECalComponentId id;
 		char *calobj = g_strdup (icalcomponent_as_ical_string (e_cal_component_get_icalcomponent (orig_value)));
 
-		e_cal_backend_notify_object_removed (E_CAL_BACKEND(wcap), uid, calobj, NULL);
+		id.uid = g_strdup (uid);
+		id.rid = NULL;
+		e_cal_backend_notify_object_removed (E_CAL_BACKEND(wcap), &id, calobj, NULL);
+		g_free (id.uid);
 		g_free (calobj);
 		g_hash_table_remove (priv->objects, uid);
 		g_free (orig_key);
@@ -583,9 +964,13 @@ cal_backend_wcap_remove_component (CalBackendWCAP *wcap, const char *uid, CalObj
 		
 		e_cal_component_get_uid (remove_comp, &mangled_uid);
 		if (g_hash_table_lookup_extended (priv->objects, mangled_uid, &orig_key, &orig_value)) {
+			ECalComponentId id;
 			char *calobj = g_strdup (icalcomponent_as_ical_string (e_cal_component_get_icalcomponent (remove_comp)));
 
-			e_cal_backend_notify_object_removed (E_CAL_BACKEND(wcap), mangled_uid, calobj, NULL);
+			id.uid = g_strdup (mangled_uid);
+			id.rid = NULL;
+			e_cal_backend_notify_object_removed (E_CAL_BACKEND(wcap), &id, calobj, NULL);
+			g_free (id.uid);
 			g_hash_table_remove (priv->objects, mangled_uid);
 			g_free (calobj);
 			g_free (orig_key);
@@ -1004,6 +1389,9 @@ cal_backend_wcap_poll_cb (gpointer data)
 	icalcomponent *icalcomp;
 	guint error_code;
 	
+	if (!priv->so_cnc)
+		return FALSE;
+
 	utc_zone = icaltimezone_get_utc_timezone ();
 
 	/* Get the latest properties */
@@ -1217,6 +1605,10 @@ cal_backend_wcap_is_read_only (ECalBackendSync *backend, EDataCal *cal, gboolean
 		return GNOME_Evolution_Calendar_Success;
 	}
 
+	if (!priv->so_cnc || !priv->props) {
+		return GNOME_Evolution_Calendar_OtherError;
+	}
+
 	hasWrite = sunone_util_has_permissions (priv->props, sunone_connection_get_user (priv->so_cnc),
                        SUNONE_ACE_CONTEXT_CALENDAR_COMPONENTS,
                        SUNONE_ACE_PERMISSION_WRITE);
@@ -1256,7 +1648,7 @@ cal_backend_wcap_get_alarm_email_address (ECalBackendSync *backend, EDataCal *ca
 
 	d(printf("cal_backend_wcap_get_alarm_email_address(%p, %s)\n", backend, *address));
 
-	if (!priv->alarm_email_address) {
+	if (!priv->alarm_email_address && priv->so_cnc) {
 		prefs = sunone_connection_get_preferences (priv->so_cnc);
 		if (prefs->alarm_mail)
 			priv->alarm_email_address = g_strdup (prefs->alarm_mail);
@@ -1324,28 +1716,40 @@ go_online (CalBackendWCAP *wcap)
 	char *key;
 	const char *uristr;
 
+	g_mutex_lock (priv->open_lock);
+
 	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
-	if (!priv->uri)
-		priv->uri = g_strdup (uristr);
-
-	if (!priv->calid)
-		priv->calid = sunone_util_get_calid_from_uri (uristr);
-
-	if (!priv->cache_name)
-		priv->cache_name = get_cache_name (wcap, TRUE);
-
-	account = sunone_component_get_account_from_uri (global_sunone_component, priv->uri);
+	account = sunone_component_get_account_from_uri (global_sunone_component, uristr);
 	if (!account) {
-		cal_backend_wcap_clean_wcap (wcap, TRUE);
-		return GNOME_Evolution_Calendar_OtherError;
+		g_mutex_unlock (priv->open_lock);
+		priv->open_error = GNOME_Evolution_Calendar_OtherError;
+		return priv->open_error;
 	}
 	cnc = sunone_account_get_connection (account);
 	if (!cnc) {
-		cal_backend_wcap_clean_wcap (wcap, TRUE);
 		g_object_unref (account);
-		return GNOME_Evolution_Calendar_OtherError;
+		g_mutex_unlock (priv->open_lock);
+		priv->open_error = GNOME_Evolution_Calendar_OtherError;
+		return priv->open_error;
 	}
+
+	if (priv->calid)
+		g_free (priv->calid);
+	priv->calid = sunone_util_get_calid_from_uri (uristr);
+	if (!priv->calid) {
+		g_object_unref (account);
+		g_mutex_unlock (priv->open_lock);
+		priv->open_error = GNOME_Evolution_Calendar_OtherError;
+		return priv->open_error;
+	}
+	if (priv->uri)
+		g_free (priv->uri);
+	priv->uri = g_strdup (uristr);
+	if (priv->cache_name)
+		g_free (priv->cache_name);
+	priv->cache_name = get_cache_name (wcap, TRUE);
 	priv->so_cnc = g_object_ref (cnc);
+	read_cache (wcap);
 
 	/* load timezones */
 	error_code = sunone_connection_get_all_timezones (priv->so_cnc, &icalcomp);
@@ -1353,7 +1757,9 @@ go_online (CalBackendWCAP *wcap)
 	if (!SUNONE_ERROR_IS_SUCCESSFUL (error_code)) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
 		g_object_unref (account);
-		return GNOME_Evolution_Calendar_OtherError;
+		g_mutex_unlock (priv->open_lock);
+		priv->open_error = GNOME_Evolution_Calendar_OtherError;
+		return priv->open_error;
 	}
 
 	add_toplevel_component (wcap, icalcomp, TRUE, FALSE);
@@ -1367,7 +1773,9 @@ go_online (CalBackendWCAP *wcap)
 	if (!priv->props) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
 		g_object_unref (account);
-		return GNOME_Evolution_Calendar_OtherError;
+		g_mutex_unlock (priv->open_lock);
+		priv->open_error = GNOME_Evolution_Calendar_OtherError;
+		return priv->open_error;
 	}
 	
 	/* Get the appropriate timezone for this calendar */
@@ -1375,7 +1783,9 @@ go_online (CalBackendWCAP *wcap)
 	if (!prefs) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
 		g_object_unref (account);
-		return GNOME_Evolution_Calendar_OtherError;
+		g_mutex_unlock (priv->open_lock);
+		priv->open_error = GNOME_Evolution_Calendar_OtherError;
+		return priv->open_error;
 	}
 
 	if (prefs->single_calendar_tzid && priv->props->timezone) {
@@ -1407,13 +1817,18 @@ go_online (CalBackendWCAP *wcap)
 	if (!SUNONE_ERROR_IS_SUCCESSFUL (error_code)) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
 		g_object_unref (account);
+		g_mutex_unlock (priv->open_lock);
 
-		if (error_code == SUNONE_ERROR_ACCESS_DENIED)
-			return GNOME_Evolution_Calendar_PermissionDenied;
-		else if (error_code == SUNONE_ERROR_LOGIN_OK_DEFAULT_CALENDAR_NOT_FOUND)
-			return GNOME_Evolution_Calendar_NoSuchCal;
-
-		return GNOME_Evolution_Calendar_OtherError;
+		if (error_code == SUNONE_ERROR_ACCESS_DENIED) {
+			priv->open_error = GNOME_Evolution_Calendar_PermissionDenied;
+			return priv->open_error;
+		} else if (error_code == SUNONE_ERROR_LOGIN_OK_DEFAULT_CALENDAR_NOT_FOUND) {
+			priv->open_error = GNOME_Evolution_Calendar_NoSuchCal;
+			return priv->open_error;
+		} else {
+			priv->open_error = GNOME_Evolution_Calendar_OtherError;
+			return priv->open_error;
+		}
 	}
 
 	add_toplevel_component (wcap, icalcomp, TRUE, FALSE);
@@ -1434,6 +1849,8 @@ go_online (CalBackendWCAP *wcap)
 
 	cal_backend_wcap_write_cache (wcap);
 
+	g_mutex_unlock (priv->open_lock);
+
 	return GNOME_Evolution_Calendar_Success;
 }
 
@@ -1443,62 +1860,55 @@ cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if
 {
 	CalBackendWCAP *wcap = (CalBackendWCAP *) backend;
 	CalBackendWCAPPrivate *priv = wcap->priv;
-	char *type;
-	const char *uristr;
-	int ret;
-	
+	icalcomponent_kind kind;
+	GThread *thread = NULL;
+	GError *error = NULL;
+
 	d(printf("cal_backend_wcap_open(%p, %d)\n", backend, only_if_exists));
 
 	/* check whether already open */
 	if (priv->uri)
 		return GNOME_Evolution_Calendar_Success;
 
-	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
-
-	g_mutex_lock (priv->open_lock);
-
-	priv->uri = g_strdup (uristr);
-	
-	priv->calid = sunone_util_get_calid_from_uri (uristr);
-	if (!priv->calid) {
-		cal_backend_wcap_clean_wcap (wcap, TRUE);
-		g_mutex_unlock (priv->open_lock);
-		return GNOME_Evolution_Calendar_NoSuchCal;
-	}
-
-	type = sunone_util_get_parameter_from_uri (uristr, "type");
-	if (type && !strcmp (type, "tasks"))
+	kind = e_cal_backend_get_kind (E_CAL_BACKEND(wcap));
+	if (kind == ICAL_VTODO_COMPONENT)
 		priv->type = TYPE_TODO;
-	else if (type && !strcmp (type, "calendar"))
+	else if (kind == ICAL_VEVENT_COMPONENT)
 		 priv->type = TYPE_EVENT;
-	g_free (type);
-
-	priv->cache_name = get_cache_name (wcap, TRUE);
 
 	if (priv->mode == CAL_MODE_LOCAL) {
 		ESource *source;
+		const char *uristr;
 		const char *display_contents = NULL;
-			
+
+		uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
+		priv->calid = sunone_util_get_calid_from_uri (uristr);
+		if (!priv->calid)
+			return GNOME_Evolution_Calendar_NoSuchCal;
+		priv->uri = g_strdup (uristr);
+		priv->cache_name = get_cache_name (wcap, TRUE);
+
 		source = e_cal_backend_get_source (E_CAL_BACKEND (backend));
 		display_contents = e_source_get_property (source, "offline_sync");
-		
 		if (!display_contents || !g_str_equal (display_contents, "1")) {
 			cal_backend_wcap_clean_wcap (wcap, TRUE);
-			g_mutex_unlock (priv->open_lock);
 			return GNOME_Evolution_Calendar_RepositoryOffline;
 		}
 
 		read_cache (wcap);
 
-		g_mutex_unlock (priv->open_lock);
 		return GNOME_Evolution_Calendar_Success;
 	}
 
-	read_cache (wcap);
-	ret = go_online (wcap);
-	g_mutex_unlock (priv->open_lock);
+	thread = g_thread_create ((GThreadFunc)go_online, wcap, FALSE, &error);
+	if (!thread) {
+		g_warning (G_STRLOC ": %s", error->message);
+		g_error_free (error);
 
-	return ret;
+		return GNOME_Evolution_Calendar_OtherError;
+	}
+
+	return GNOME_Evolution_Calendar_Success;
 }
 
 static ECalBackendSyncStatus
@@ -1660,7 +2070,6 @@ cal_backend_wcap_get_object_list (ECalBackendSync *backend, EDataCal *cal,
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	struct search_data match_data;
 	const char *uristr;
-	char *type;
 
 	g_return_val_if_fail (sexp != NULL, GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (objects != NULL, GNOME_Evolution_Calendar_InvalidObject);
@@ -1680,14 +2089,11 @@ cal_backend_wcap_get_object_list (ECalBackendSync *backend, EDataCal *cal,
 
 	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (backend));
 
-	type = sunone_util_get_parameter_from_uri (uristr, "type");
 	g_hash_table_foreach (priv->objects, match_object, &match_data);
 
 	g_object_unref (match_data.sexp);
 	*objects = match_data.matches;
 
-	g_free (type);
-
 	return GNOME_Evolution_Calendar_Success;
 }
 
@@ -1853,8 +2259,9 @@ cal_backend_wcap_get_changes (ECalBackendSync *backend, EDataCal *cal, const cha
 	CalBackendWCAP *wcap = CAL_BACKEND_WCAP (backend);
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	SunOneAccount *account;
+	icalcomponent_kind kind;
 	const char *uristr;
-	char *dirname, *filename, *type;
+	char *dirname, *filename;
 	EXmlHash *xmlhash;
 	struct ChangeData data;
 	struct cbw_data cbwdata;
@@ -1867,18 +2274,18 @@ cal_backend_wcap_get_changes (ECalBackendSync *backend, EDataCal *cal, const cha
 
 	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (backend));
 
-	type = sunone_util_get_parameter_from_uri (uristr, "type");
+	kind = e_cal_backend_get_kind (E_CAL_BACKEND (backend));
 
 	/* open the changes file */
 	account = sunone_component_get_account_from_uri (global_sunone_component, priv->uri);
 	if (!account)
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 	
-	if (type && !strcmp(type, "calendar"))
+	if (kind ==  E_CAL_COMPONENT_EVENT)
 		dirname = g_strdup_printf ("%s/sunone/%s@%s/Calendar", evolution_dir,
 					   sunone_account_get_user (account),
 					   sunone_account_get_server (account));
-	else if (type && !strcmp(type, "tasks"))
+	else if (kind == E_CAL_COMPONENT_TODO)
 		dirname = g_strdup_printf ("%s/sunone/%s@%s/Tasks", evolution_dir,
 					   sunone_account_get_user (account),
 					   sunone_account_get_server (account));
@@ -1906,7 +2313,7 @@ cal_backend_wcap_get_changes (ECalBackendSync *backend, EDataCal *cal, const cha
 	data.ehash = xmlhash;
 	data.adds = NULL;
 	data.modifies = NULL;
-	if (type && (!strcmp(type, "calendar") || !strcmp(type, "tasks")))
+	if (kind == E_CAL_COMPONENT_EVENT || kind == E_CAL_COMPONENT_TODO)
 		g_hash_table_foreach (priv->objects, check_change_type, &data);
 	else
 		return GNOME_Evolution_Calendar_OtherError;
@@ -1931,8 +2338,6 @@ cal_backend_wcap_get_changes (ECalBackendSync *backend, EDataCal *cal, const cha
 
 	e_xmlhash_destroy (xmlhash);
 
-	g_free (type);
-
 	return GNOME_Evolution_Calendar_Success;
 }
 
@@ -2029,9 +2434,11 @@ cal_backend_wcap_set_mode (ECalBackend *backend, CalMode mode)
 	if (mode == CAL_MODE_REMOTE) {
 		if (priv->uri) {
 			account = sunone_component_get_account_from_uri (global_sunone_component, priv->uri);
-			sunone_account_set_online (account);
-			status = go_online (wcap);
-			g_object_unref (account);
+			if (account) {
+				sunone_account_set_online (account);
+				status = go_online (wcap);
+				g_object_unref (account);
+			}
 		}
 	} else if (mode == CAL_MODE_LOCAL) {
 		if (priv->so_cnc && sunone_connection_is_open (priv->so_cnc))
@@ -2039,8 +2446,10 @@ cal_backend_wcap_set_mode (ECalBackend *backend, CalMode mode)
 		in_offline (wcap);
 		if (priv->uri) {
 			account = sunone_component_get_account_from_uri (global_sunone_component, priv->uri);
-			sunone_account_set_offline (account);
-			g_object_unref (account);
+			if (account) {
+				sunone_account_set_offline (account);
+				g_object_unref (account);
+			}
 		}
 	} else {
 		e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_NOT_SUPPORTED, cal_mode_to_corba (mode));
@@ -2097,6 +2506,14 @@ cal_backend_wcap_result_from_error (guint error_code)
 	return GNOME_Evolution_Calendar_InvalidObject;
 }
 
+ECalBackendSyncStatus
+cal_backend_wcap_get_open_error (CalBackendWCAP *wcap)
+{
+	g_return_val_if_fail (IS_CAL_BACKEND_WCAP (wcap), GNOME_Evolution_Calendar_OtherError);
+
+	return wcap->priv->open_error;
+}
+
 static ECalBackendSyncStatus
 cal_backend_wcap_get_timezone (ECalBackendSync *backend, EDataCal *cal,
 			            const char *tzid, char **object)
@@ -2177,6 +2594,7 @@ cal_backend_wcap_init (CalBackendWCAP *object)
 
 	priv->type = TYPE_ALL;
 	priv->mode = CAL_MODE_REMOTE;
+	priv->open_error = GNOME_Evolution_Calendar_OtherError;
 
 	priv->timeout_id = -1;
 
