@@ -19,6 +19,7 @@
 #include <libsoup/soup.h>
 #include <libedataserver/e-time-utils.h>
 
+#include "sunone-util.h"
 #include "sunone-connection.h"
 #include "sunone-message.h"
 #include "calendar/cal-backend-wcap.h"
@@ -62,7 +63,6 @@ static guint sunone_connection_storeevents_by_mail (SunOneConnection *connection
 
 static GObjectClass *parent_class = NULL;
 
-
 static char *
 query_encode (const char *part, gboolean escape_unsafe,
 	      const char *escape_extra)
@@ -615,16 +615,16 @@ sunone_connection_init (SunOneConnection *object)
 
 
 static gboolean
-parse_session_object (SunOneConnection *connection, const char *body)
+parse_session_object (SunOneConnection *connection, SoupMessage *msg)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
 	icalcomponent *icalcomp;
 	icalcomponent_kind kind;
 	icalproperty *icalprop;
 
-	g_return_val_if_fail (body != NULL, FALSE);
+	g_return_val_if_fail (msg != NULL, FALSE);
 
-	icalcomp = icalparser_parse_string (body);
+	icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 	if (!icalcomp)
 		return FALSE;
 
@@ -690,7 +690,7 @@ sunone_login (SunOneConnection *connection, gboolean refresh)
 
 	error_code = msg->status_code;
 	if (SUNONE_ERROR_IS_SUCCESSFUL (error_code)) {
-		if (parse_session_object (connection, msg->response.body)) {
+		if (parse_session_object (connection, msg)) {
 			if (priv->timeout_id == -1)
 				priv->timeout_id = g_timeout_add (REFRESH_INTERVAL,
 									(GSourceFunc) reconnect_timeout_cb,
@@ -1086,7 +1086,7 @@ sunone_connection_get_preferences (SunOneConnection *connection)
 	}
 
 	/* parse the returned data */
-	icalcomp = icalparser_parse_string (msg->response.body);
+	icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 	g_object_unref (msg);
 	if (!icalcomp) {
 		g_static_mutex_unlock (&mutex);
@@ -1379,7 +1379,7 @@ sunone_connection_createcalendar (SunOneConnection *connection, const gchar *cal
 		icalcomponent *icalcomp;
 
 		/* parse the response to get the calid as generated in the server */
-		icalcomp = icalparser_parse_string (msg->response.body);
+		icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 		if (icalcomp) {
 			icalproperty *icalprop;
 
@@ -1627,7 +1627,7 @@ sunone_connection_get_calprops (SunOneConnection *connection, const gchar *calid
 		}
 
 		/* parse the returned data */
-		icalcomp = icalparser_parse_string (msg->response.body);
+		icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 		g_object_unref (msg);
 		if (!icalcomp) {
 			g_static_mutex_unlock (&mutex);
@@ -1796,7 +1796,7 @@ sunone_connection_list_calids (SunOneConnection *connection)
 	soup_uri_free (get_uri);
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		icalcomp = icalparser_parse_string (msg->response.body);
+		icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 		if (icalcomp) {
 			subcomp = icalcomponent_get_first_component (icalcomp, ICAL_VCALENDAR_COMPONENT);
 			while (subcomp) {
@@ -1859,7 +1859,7 @@ sunone_connection_search_calids (SunOneConnection *connection, SunOneSearchField
 	soup_uri_free (get_uri);
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		icalcomp = icalparser_parse_string (msg->response.body);
+		icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 		if (icalcomp) {
 			subcomp = icalcomponent_get_first_component (icalcomp, ICAL_VCALENDAR_COMPONENT);
 			while (subcomp) {
@@ -1926,7 +1926,7 @@ sunone_connection_get_all_timezones (SunOneConnection *connection,
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
+		*icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 		if (!*icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -2085,7 +2085,7 @@ sunone_connection_fetchcomponents_by_lastmod (SunOneConnection *connection,
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
+		*icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 		if (!*icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 
@@ -2141,7 +2141,7 @@ sunone_connection_fetch_deletedcomponents (SunOneConnection *connection,
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
+		*icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 		if (!*icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -2183,7 +2183,7 @@ sunone_connection_get_freebusy (SunOneConnection *connection,
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
+		*icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 		if (!*icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -2210,7 +2210,7 @@ sunone_connection_get_freebusy (SunOneConnection *connection,
 		retval = msg->status_code;
 
 		if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-			*icalcomp = icalparser_parse_string (msg->response.body);
+			*icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 			if (!*icalcomp)
 				retval = SOUP_STATUS_MALFORMED;
 		}
@@ -2510,7 +2510,7 @@ sunone_connection_storeevents_by_mail (SunOneConnection *connection, const char
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code) && return_icalcomp) {
-		*return_icalcomp = icalparser_parse_string (msg->response.body);
+		*return_icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 		if (!*return_icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -2740,7 +2740,7 @@ sunone_connection_storetodos (SunOneConnection *connection, const char *calid, i
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code) && return_icalcomp) {
-		*return_icalcomp = icalparser_parse_string (msg->response.body);
+		*return_icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 		if (!*return_icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -2795,7 +2795,7 @@ sunone_connection_verifyevents_by_ids (SunOneConnection *connection,
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
+		*icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 		if (!*icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -2847,7 +2847,7 @@ sunone_connection_verifytodos_by_ids (SunOneConnection *connection,
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
+		*icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 		if (!*icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -2858,90 +2858,6 @@ sunone_connection_verifytodos_by_ids (SunOneConnection *connection,
 	return retval;
 }
 
-#if 0
-guint
-sunone_connection_fetchevents_by_id (SunOneConnection *connection,
-				      const char *calid,
-				      const char *uid,
-//				      const char **ridv,
-					  SunOneModType mod,
-				      icalcomponent **icalcomp)
-{
-	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *fetch_uri;
-	SoupMessage *msg;
-	guint retval;
-
-	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), SOUP_STATUS_CANCELLED);
-
-	IS_CONNECTED (connection, SOUP_STATUS_CANT_CONNECT);
-	
-	/* prepare the URI */
-	NEW_URI (fetch_uri);
-	fetch_uri->path = g_strdup ("/fetchevents_by_id.wcap");
-	fetch_uri->query = g_strdup_printf (
-		"id=%s&calid=%s&fmt-out=text/calendar&uid=%s&mod=%d&compressed=1",
-		priv->wcap_session_id,
-		calid, uid, mod);
-
-	msg = sunone_message_new_from_uri (fetch_uri, SOUP_METHOD_GET);
-	sunone_message_send (connection, msg);
-	retval = msg->status_code;
-
-	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
-		if (!*icalcomp)
-			retval = SOUP_STATUS_MALFORMED;
-	}
-
-	g_object_unref (msg);
-	soup_uri_free (fetch_uri);
-
-	return retval;
-}
-
-guint
-sunone_connection_fetchtodos_by_id (SunOneConnection *connection,
-				      const char *calid,
-				      const char *uid,
-//				      const char **ridv,
-					  SunOneModType mod,
-				      icalcomponent **icalcomp)
-{
-	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *fetch_uri;
-	SoupMessage *msg;
-	guint retval;
-
-	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), SOUP_STATUS_CANCELLED);
-
-	IS_CONNECTED (connection, SOUP_STATUS_CANT_CONNECT);
-	
-	/* prepare the URI */
-	NEW_URI (fetch_uri);
-	fetch_uri->path = g_strdup ("/fetchtodos_by_id.wcap");
-	fetch_uri->query = g_strdup_printf (
-		"id=%s&calid=%s&fmt-out=text/calendar&uid=%s&mod=%d",
-		priv->wcap_session_id,
-		calid, uid, mod);
-
-	msg = sunone_message_new_from_uri (fetch_uri, SOUP_METHOD_GET);
-	sunone_message_send (connection, msg);
-	retval = msg->status_code;
-
-	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
-		if (!*icalcomp)
-			retval = SOUP_STATUS_MALFORMED;
-	}
-
-	g_object_unref (msg);
-	soup_uri_free (fetch_uri);
-
-	return retval;
-}
-#endif
-	
 void
 sunone_connection_add_wcap (SunOneConnection *connection, const char *key, gpointer wcap)
 {
@@ -2988,7 +2904,7 @@ sunone_connection_version (SunOneConnection *connection)
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		icalcomp = icalparser_parse_string (msg->response.body);
+		icalcomp = sunone_util_icalparser_parse_string (msg->response.body, msg->response.length);
 		if (!icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 		else {
