@@ -48,6 +48,7 @@ static GType col_types[] = {
 	G_TYPE_POINTER,
 	G_TYPE_STRING,
 	G_TYPE_STRING,
+	G_TYPE_STRING,
 	G_TYPE_UINT,
 	G_TYPE_BOOLEAN,
 	G_TYPE_BOOLEAN
@@ -130,6 +131,7 @@ sunone_folder_tree_view_new (SunOneFolderTree *tree, const char *physical_uri)
 
 	widget = sunone_invitation_list_new (model);
 	gtk_widget_show (widget);
+	gtk_widget_ref (widget);
 
 	return widget;
 }
@@ -247,115 +249,53 @@ add_folder_tree_item (SunOneFolderTree *tree, SunOneAccount *account, GtkTreeIte
 	SunOneInvitationListModel *model;
 	char *full_name;
 	char *physical_uri;
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
 	
 	g_return_if_fail (SUNONE_IS_FOLDER_TREE (tree));
 	g_return_if_fail (account != NULL);
 
 	/* create the invitation list model now */
-	physical_uri = sunone_util_get_physical_uri_from_account (account, calid, "invitations");
+	physical_uri = sunone_account_get_physical_uri (account, calid, "invitations");
+	g_static_mutex_lock (&mutex);
 	model = SUNONE_INVITATION_LIST_MODEL (sunone_invitation_list_model_new (physical_uri));
+	g_static_mutex_unlock (&mutex);
 	g_free (physical_uri);
 	g_return_if_fail (model != NULL);
 	priv->invitation_models = g_list_prepend (priv->invitation_models, model);
 
-	physical_uri = sunone_util_get_physical_uri_from_account (account, calid, "folder");
+	full_name = g_strdup_printf ("/%s/%s/", sunone_account_get_name (account), calid);
+	physical_uri = sunone_account_get_physical_uri (account, calid, "folder");
 	gtk_tree_store_append (priv->treeview_model, &iter1, iter);
 	gtk_tree_store_set (priv->treeview_model, &iter1,
 				COL_STRING_DISPLAY_NAME, display_name,
 				COL_POINTER_ACCOUNT, account,
 				COL_STRING_FOLDER_TYPE, "folder",
 				COL_STRING_URI, physical_uri,
+				COL_STRING_FULLNAME, full_name,
 				COL_BOOL_IS_STORE, TRUE, 
 				COL_BOOL_IS_ACCOUNT, FALSE,
 				COL_BOOL_IS_PERSONAL, personal,
 				-1);
 	g_free (physical_uri);
-	full_name = g_strdup_printf ("/%s/%s/", sunone_account_get_name (account), calid);
 	full_hash_insert (tree, &iter1, full_name);
 	g_free (full_name);
 
-	physical_uri = sunone_util_get_physical_uri_from_account (account, calid, "invitations");
+	full_name = g_strdup_printf ("/%s/%s/invitations", sunone_account_get_name (account), calid);
+	physical_uri = sunone_account_get_physical_uri (account, calid, "invitations");
 	gtk_tree_store_append (priv->treeview_model, &iter2, &iter1);
 	gtk_tree_store_set (priv->treeview_model, &iter2,
 				COL_STRING_DISPLAY_NAME, "Invitations",
 				COL_POINTER_ACCOUNT, account,
 				COL_STRING_FOLDER_TYPE, "invitations",
 				COL_STRING_URI, physical_uri,
+				COL_STRING_FULLNAME, full_name,
 				COL_BOOL_IS_STORE, FALSE, 
 				COL_BOOL_IS_ACCOUNT, FALSE,
 				COL_BOOL_IS_PERSONAL, FALSE,
 				-1); 
 	g_free (physical_uri);
-	full_name = g_strdup_printf ("/%s/%s/invitations", sunone_account_get_name (account), calid);
 	full_hash_insert (tree, &iter2, full_name);
 	g_free (full_name);
-#if 0
-	if (personal) {
-		physical_uri = sunone_util_get_physical_uri_from_account (account, calid, "folder");
-		gtk_tree_store_append (priv->treeview_model, &iter1, iter);
-		gtk_tree_store_set (priv->treeview_model, &iter1,
-					COL_STRING_DISPLAY_NAME, _("Personal"),
-					COL_POINTER_ACCOUNT, account,
-					COL_STRING_FOLDER_TYPE, "folder",
-					COL_STRING_URI, physical_uri,
-					COL_BOOL_IS_STORE, TRUE, 
-					COL_BOOL_IS_ACCOUNT, FALSE,
-					COL_BOOL_IS_PERSONAL, TRUE,
-					-1);
-		g_free (physical_uri);
-		full_name = g_strdup_printf ("/%s/%s/", sunone_account_get_name (account), calid);
-		full_hash_insert (tree, &iter1, full_name);
-		g_free (full_name);
-
-		physical_uri = sunone_util_get_physical_uri_from_account (account, calid, "invitations");
-		gtk_tree_store_append (priv->treeview_model, &iter2, &iter1);
-		gtk_tree_store_set (priv->treeview_model, &iter2,
-					COL_STRING_DISPLAY_NAME, _("Personal Invitations"),
-					COL_POINTER_ACCOUNT, account,
-					COL_STRING_FOLDER_TYPE, "invitations",
-					COL_STRING_URI, physical_uri,
-					COL_BOOL_IS_STORE, FALSE, 
-					COL_BOOL_IS_ACCOUNT, FALSE,
-					COL_BOOL_IS_PERSONAL, FALSE,
-					-1);
-		g_free (physical_uri);
-		full_name = g_strdup_printf ("/%s/invitations", sunone_account_get_name (account));
-		full_hash_insert (tree, &iter1, full_name);
-		g_free (full_name);
-	} else {
-		physical_uri = sunone_util_get_physical_uri_from_account (account, calid, "folder");
-		gtk_tree_store_append (priv->treeview_model, &iter1, iter);
-		gtk_tree_store_set (priv->treeview_model, &iter1,
-					COL_STRING_DISPLAY_NAME, display_name,
-					COL_POINTER_ACCOUNT, account,
-					COL_STRING_FOLDER_TYPE, "folder",
-					COL_STRING_URI, physical_uri,
-					COL_BOOL_IS_STORE, TRUE, 
-					COL_BOOL_IS_ACCOUNT, FALSE,
-					COL_BOOL_IS_PERSONAL, FALSE,
-					-1);
-		g_free (physical_uri);
-		full_name = g_strdup_printf ("/%s/%s/", sunone_account_get_name (account), calid);
-		full_hash_insert (tree, &iter1, full_name);
-		g_free (full_name);
-
-		physical_uri = sunone_util_get_physical_uri_from_account (account, calid, "invitations");
-		gtk_tree_store_append (priv->treeview_model, &iter2, &iter1);
-		gtk_tree_store_set (priv->treeview_model, &iter2,
-					COL_STRING_DISPLAY_NAME, "Invitations",
-					COL_POINTER_ACCOUNT, account,
-					COL_STRING_FOLDER_TYPE, "invitations",
-					COL_STRING_URI, physical_uri,
-					COL_BOOL_IS_STORE, FALSE, 
-					COL_BOOL_IS_ACCOUNT, FALSE,
-					COL_BOOL_IS_PERSONAL, FALSE,
-					-1); 
-		g_free (physical_uri);
-		full_name = g_strdup_printf ("/%s/%s/invitations", sunone_account_get_name (account), calid);
-		full_hash_insert (tree, &iter2, full_name);
-		g_free (full_name);
-	}
-#endif
 }
 
 static void
@@ -393,24 +333,8 @@ sunone_folder_tree_connect (SunOneFolderTree *tree, SunOneAccount *account,
 	g_return_if_fail (SUNONE_IS_FOLDER_TREE (tree));
 
 	cnc = sunone_account_get_connection (account);
-	if (!cnc) {
-		const char *hname;
-#ifdef ENABLE_IDN
-		const char *t;
-		t = sunone_account_get_server (account);
-		hname = (const char *) sunone_util_get_decoded_server_port_name (t);
-#else
-		hname = sunone_account_get_server (account);
-#endif
-		/* Check whether user cancelled the password dialog, 
-		 * and if so, no need to show the error dialog */
-		if (!sunone_account_get_dont_ask_password (account)) {
-			e_notice (NULL, GTK_MESSAGE_ERROR,
-				  _("Could not create connection for %s"), hname);
-		}
+	if (!cnc)
 		return;
-	}
-
 
 	prefs = sunone_connection_get_preferences (cnc);
 	if (!prefs)
@@ -424,48 +348,15 @@ sunone_folder_tree_connect (SunOneFolderTree *tree, SunOneAccount *account,
 	sunone_connection_free_preferences (prefs);
 }
 
-void
-sunone_folder_tree_created (SunOneFolderTree *tree, SunOneAccount *account)
-{
-	SunOneFolderTreePrivate *priv = tree->priv;
-	GtkTreeIter iter;
-	char *physical_uri;
-	char *full_name;
-	int mode;
-
-	g_return_if_fail (SUNONE_IS_FOLDER_TREE (tree));
-	g_return_if_fail (account != NULL);
-
-	physical_uri = sunone_util_get_physical_uri_from_account (account, sunone_account_get_user (account), "folder");
-	gtk_tree_store_append (priv->treeview_model, &iter, NULL);
-	gtk_tree_store_set (priv->treeview_model, &iter,
-				COL_STRING_DISPLAY_NAME, sunone_account_get_name (account),
-				COL_POINTER_ACCOUNT, account,
-				COL_STRING_FOLDER_TYPE, "folder",
-				COL_STRING_URI, physical_uri,
-				COL_BOOL_IS_STORE, TRUE, 
-				COL_BOOL_IS_ACCOUNT, TRUE,
-				COL_BOOL_IS_PERSONAL, FALSE,
-				-1);
-	g_free (physical_uri);
-
-	full_name = g_strdup_printf ("/%s/", sunone_account_get_name (account));
-	full_hash_insert (tree, &iter, full_name);
-	g_free (full_name);
-
-	sunone_account_is_offline (account, &mode);
-	/*only connect if we are online */
-	if (mode == ONLINE_MODE)
-		sunone_folder_tree_connect (tree, account, &iter);
-}
-
 static void
 remove_calendar (SunOneFolderTree *tree, const char *full_name)
 {
 	SunOneFolderTreePrivate *priv = tree->priv;
+	GtkTreeStore *model;
 	GtkTreeRowReference *uri_row;
 	GtkTreePath *path;
 	GtkTreeIter iter;
+	GtkTreeIter child;
 
 	g_return_if_fail (SUNONE_IS_FOLDER_TREE (tree));
 
@@ -473,22 +364,123 @@ remove_calendar (SunOneFolderTree *tree, const char *full_name)
 	if (!uri_row)
 		return;
 
+	model = priv->treeview_model;
 	path = gtk_tree_row_reference_get_path (uri_row);
-	gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->treeview_model), &iter, path);
-	gtk_tree_store_remove (priv->treeview_model, &iter);
+	gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
+	/*remove all children*/
+	gtk_tree_model_iter_nth_child(GTK_TREE_MODEL (model), &child, &iter, 0);
+	while (gtk_tree_store_iter_is_valid (model, &child)) {
+		char *child_full_name;
 
+		gtk_tree_model_get (GTK_TREE_MODEL (model), &child, COL_STRING_FULLNAME, &child_full_name, -1);
+		gtk_tree_model_iter_next (GTK_TREE_MODEL (model), &child);
+		remove_calendar (tree, child_full_name);
+		g_hash_table_remove (full_hash, child_full_name);
+		g_free (child_full_name);
+	}
+	gtk_tree_store_remove (model, &iter);
 	gtk_tree_path_free (path);
 	g_hash_table_remove (full_hash, full_name);
 }
 
+typedef struct 
+{
+	SunOneFolderTree *tree;
+	SunOneAccount *account;
+	GtkTreeIter iter;
+} TreeCreateData;
+
+static void
+create_folder_tree_with_thread (TreeCreateData *data)
+{
+	SunOneFolderTreePrivate *priv = data->tree->priv;
+	SunOneAccount *account = data->account;
+	GtkTreeIter iter = data->iter;
+	GtkTreeIter sub_iter;
+	GtkTreePath *path;
+
+	gtk_tree_store_append (priv->treeview_model, &sub_iter, &iter);
+	gtk_tree_store_set (priv->treeview_model, &sub_iter,
+				COL_STRING_DISPLAY_NAME, _("Connecting..."),
+				COL_POINTER_ACCOUNT, NULL,
+				COL_STRING_FOLDER_TYPE, "",
+				COL_STRING_URI, NULL,
+				COL_STRING_FULLNAME, NULL,
+				COL_BOOL_IS_STORE, TRUE, 
+				COL_BOOL_IS_ACCOUNT, FALSE,
+				COL_BOOL_IS_PERSONAL, FALSE,
+				-1);
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (priv->treeview_model), &iter);
+	gtk_tree_view_expand_row (priv->treeview, path, FALSE);
+	gtk_tree_path_free (path);
+
+	sunone_folder_tree_connect (data->tree, account, &iter);
+
+	gtk_tree_store_remove (priv->treeview_model, &sub_iter);
+	g_object_unref (data->account);
+	g_free (data);
+
+	sunone_folder_tree_sync ();
+}
+
+void
+sunone_folder_tree_created (SunOneFolderTree *tree, SunOneAccount *account)
+{
+	SunOneFolderTreePrivate *priv;
+	TreeCreateData *data;
+	GtkTreeIter iter;
+	GThread *thread = NULL;
+	GError *error = NULL;
+	char *physical_uri;
+	char *full_name;
+	int mode;
+
+	g_return_if_fail (SUNONE_IS_FOLDER_TREE (tree));
+	g_return_if_fail (account != NULL);
+
+	priv = tree->priv;
+	/*remove old tree, if exists*/
+	full_name = g_strdup_printf ("/%s/", sunone_account_get_name (account));
+	remove_calendar (tree, full_name);
+
+	physical_uri = sunone_account_get_physical_uri (account, sunone_account_get_user (account), "folder");
+	gtk_tree_store_append (priv->treeview_model, &iter, NULL);
+	gtk_tree_store_set (priv->treeview_model, &iter,
+				COL_STRING_DISPLAY_NAME, sunone_account_get_name (account),
+				COL_POINTER_ACCOUNT, account,
+				COL_STRING_FOLDER_TYPE, "folder",
+				COL_STRING_URI, physical_uri,
+				COL_STRING_FULLNAME, full_name,
+				COL_BOOL_IS_STORE, TRUE, 
+				COL_BOOL_IS_ACCOUNT, TRUE,
+				COL_BOOL_IS_PERSONAL, FALSE,
+				-1);
+	g_free (physical_uri);
+
+	full_hash_insert (tree, &iter, full_name);
+	g_free (full_name);
+
+	/*only connect if we are online */
+	sunone_account_is_offline (account, &mode);
+	if (mode == ONLINE_MODE) {
+		data = g_malloc (sizeof (TreeCreateData));
+		data->tree = tree;
+		data->account = g_object_ref (account);
+		data->iter = iter;
+		thread = g_thread_create ((GThreadFunc)create_folder_tree_with_thread, data, FALSE, &error);
+		if (!thread) {
+			g_warning (G_STRLOC ": %s", error->message);
+			g_error_free (error);
+			g_object_unref (data->account);
+			g_free (data);
+		}
+	}
+}
+
 void
 sunone_folder_tree_removed (SunOneFolderTree *tree, SunOneAccount *account)
 {
 	SunOneFolderTreePrivate *priv = tree->priv;
-	SunOneCalendarSubscription *sub;
-	SunOneConnectionPreferences *prefs;
-	SunOneConnection *cnc;
-	GList *l;
 	char *full_name;
 	GtkWidget *page;
 	int page_num;
@@ -499,38 +491,7 @@ sunone_folder_tree_removed (SunOneFolderTree *tree, SunOneAccount *account)
 	remove_calendar (tree, full_name);
 	g_free (full_name);
 
-	cnc = sunone_account_get_connection (account);
-	if (!cnc)
-		return;
-
-	prefs = sunone_connection_get_preferences (cnc);
-	if (!prefs)
-		return;
-
-	/* create the folders as per the subscription list */
-	for (l = prefs->subscriptions; l != NULL; l = l->next) {
-		sub = (SunOneCalendarSubscription *) l->data;
-		if (!sub)
-			continue;
-
-		if (strcasecmp (sub->calid, prefs->default_calendar)) {
-			full_name = g_strdup_printf ("/%s/%s/invitations", sunone_account_get_name (account), sub->calid);
-			remove_calendar (tree, full_name);
-			g_free (full_name);
-
-			full_name = g_strdup_printf ("/%s/%s/", sunone_account_get_name (account), sub->calid);
-			remove_calendar (tree, full_name);
-			g_free (full_name);
-
-		}
-	}
-	
-	full_name = g_strdup_printf ("/%s/invitations", sunone_account_get_name (account));
-	remove_calendar (tree, full_name);
-	g_free (full_name);
-
 	remove_source (account, sunone_account_get_user (account), TRUE);
-	sunone_connection_free_preferences (prefs);
 
 	page = sunone_folder_tree_no_view ();
 	page_num = gtk_notebook_page_num (priv->body, page);
@@ -550,10 +511,6 @@ sunone_folder_tree_remove_calendar (SunOneFolderTree *tree, SunOneAccount *accou
 	g_return_if_fail (SUNONE_IS_FOLDER_TREE (tree));
 	g_return_if_fail (account != NULL);
 
-	full_name = g_strdup_printf ("/%s/%s/invitations", sunone_account_get_name (account), calid);
-	remove_calendar (tree, full_name);
-	g_free (full_name);
-
 	full_name = g_strdup_printf ("/%s/%s/", sunone_account_get_name (account), calid);
 	remove_calendar (tree, full_name);
 	g_free (full_name);
@@ -598,10 +555,6 @@ sunone_folder_tree_rename_calendar (SunOneFolderTree *tree, SunOneAccount *accou
 	g_return_if_fail (account != NULL);
 
 	/* remove old folder */
-	full_name = g_strdup_printf ("/%s/%s/invitations", sunone_account_get_name (account), calid);
-	remove_calendar (tree, full_name);
-	g_free (full_name);
-
 	full_name = g_strdup_printf ("/%s/%s/", sunone_account_get_name (account), calid);
 	remove_calendar (tree, full_name);
 	g_free (full_name);
@@ -1269,10 +1222,9 @@ sft_popup_login (EPopup *ep, EPopupItem *item, void *data)
 		return;
 
 	/* recreate the folder list */
-	sunone_folder_tree_removed (tree, account);
 	sunone_account_set_dont_ask_password (account, FALSE);
+	sunone_account_forget_password (account);
 	sunone_folder_tree_created (tree, account);
-	sunone_folder_tree_sync ();
 }
 
 static EPopupItem sft_popup_menu[] = {
@@ -1308,8 +1260,6 @@ sft_tree_popup (SunOneFolderTree *tree, GdkEvent *event)
 	gboolean is_store, is_account, is_personal;
 	guint32 flags = ~0ul;
 
-	account = sunone_folder_tree_get_account (tree);
-
 	selection = gtk_tree_view_get_selection (tree->priv->treeview);
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
 		return FALSE; 
